# encoding: utf-8
"""
pyDGS - a Python framework for wavelet-based digital grain size analysis

pyDGS is an open-source project dedicated to provide a Python framework to compute estimates of grain size distribution  using the continuous wavelet transform method of Buscombe (2013) from an image of sediment where grains are clearly resolved. DOES NOT REQUIRE CALIBRATION

This program implements the algorithm of:

Buscombe, D. (2013) Transferable Wavelet Method for Grain-Size Distribution from Images of Sediment Surfaces and Thin Sections, and Other Natural Granular Patterns. Sedimentology 60, 1709-1732

 Author:  Daniel Buscombe
           Northern Arizona University
           Flagstaff, AZ 86001
           daniel.buscombe@nau.edu
 Revision Dec 21, 2017
 First Revision January 18 2013

For more information visit https://github.com/dbuscombe-usgs/pyDGS

https://www.danielbuscombe.com/s/Buscombe_2013_sedimentology_101111-sed12049.pdf

:install:
    python setup.py install
    sudo python setup.py install
    pip install pyDGS
    
:test:
    python -c "import DGS; DGS.test.dotest_web()"

    python
    import DGS

    image_file = '/home/sed_images/my_image.png'

    density = 10 # process every 10 lines
    resolution = 0.01 # mm/pixel
    dofilter =1 # filter the image
    notes = 8 # notes per octave
    maxscale = 8 #Max scale as inverse fraction of data length
    verbose = 1 # print stuff to screen
    x = area-number to volume-number conversion (or empirical correction)
    dgs_stats = DGS.dgs(image_file, density, resolution, dofilter, maxscale, notes, verbose, x)

 REQUIRED INPUTS:
 simply a single file path
 
 OPTIONAL INPUTS [default values][range of acceptable values]
 density = process every *density* lines of image [10][1 - 100]
 resolution = spatial resolution of image in mm/pixel [1][>0]
 dofilter = spatial resolution of image in mm/pixel [1][0 or 1]
 notes = notes per octave to consider in continuous wavelet transform [8][1 - 8]
 maxscale = maximum scale (pixels) as an inverse function of data (image row) length [8][2 - 40]
 verbose = if 1, print stuff to screen [0][0 or 1]
 x = area-by-number to volume-by-number conversion [0] [-1 - 1]

OUTPUT:
A dictionary objects containing the following key/value pairs:
* mean grain size: arithmetic mean grain size
* grain size sorting: arithmetic standard deviation of grain sizes
* grain size skewness: arithmetic skewness of grain size-distribution
* grain size kurtosis: arithmetic kurtosis of grain-size distribution
* percentiles: 5th, 10th, 16th, 25th, 50th, 75th, 84th, 90th, and 95th percentile of the cumulative grain size (% less than) particle size distribution
* grain size frequencies: the normalised frequencies associated with 'grain size bins'
* grain size bins: grain size values at which the distribution is evaluated


PROCESSING NOTES:
Note that the larger the density parameter, the longer the execution time. 

:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
    
    This software is in the public domain because it contains materials that
    originally came from the United States Geological Survey, an agency of the
    United States Department of Interior. For more information, 
    see the official USGS copyright policy at
    http://www.usgs.gov/visual-id/credit_usgs.html#copyright
    Any use of trade, product, or firm names is for descriptive purposes only 
    and does not imply endorsement by the U.S. government.
    
"""

from __future__ import print_function
import numpy as np
import sys #, getopt, os, glob
#from PIL.Image import open as imopen
import cwt
import sgolay
try:
   from imread import imread
except:
   from scipy.misc import imread

from scipy.ndimage.interpolation import zoom

# suppress divide and invalid warnings
np.seterr(divide='ignore')
np.seterr(invalid='ignore')

import warnings
warnings.filterwarnings("ignore")

# =========================================================
def iseven(n):
   """Return true if n is even."""
   return n%2==0

# =========================================================
def isodd(n):
   """Return true if n is odd."""   
   return not iseven(n)

# =========================================================
def rescale(dat,mn,mx):
    """
    rescales an input dat between mn and mx
    """
    m = min(dat.flatten())
    M = max(dat.flatten())
    return (mx-mn)*(dat-m)/(M-m)+mn

# =========================================================
def get_me(useregion, maxscale, notes, density): #, mult):
   complete=0
   while complete==0:
      try:
         dat = cwt.Cwt(np.asarray(useregion,'int8'), maxscale, notes, density) #, mult)
         if 'dat' in locals(): 
            complete=1
      except:
         density = density +1

   return dat.getvar(), (np.pi/2)*dat.getscales()

# =========================================================
def filter_me(region):

   region = zoom(region, 0.5)

   nx, ny = np.shape(region)
   mn = min(nx,ny)

   if isodd(mn/4):
        window_size = (int(mn/4))
   else:
        window_size = (int(mn/4))-1

   if iseven(window_size):
      window_size = window_size+1

   Zf = sgolay.sgolay2d( region, window_size, order=3).getdata()
   # rescale filtered image to full 8-bit range
   useregion = rescale(zoom(region-Zf[:nx,:ny],2),0,255)

   return useregion

# =========================================================
# =========================================================
def dgs(image, density=10, resolution=1, dofilter=1, maxscale=8, notes=8, verbose=0, x=0):

   if verbose==1:
      print("===========================================")
      print("======DIGITAL GRAIN SIZE: WAVELET==========")
      print("===========================================")
      print("=CALCULATE GRAIN SIZE-DISTRIBUTION FROM AN=")
      print("====IMAGE OF SEDIMENT/GRANULAR MATERIAL====")
      print("===========================================")
      print("======A PROGRAM BY DANIEL BUSCOMBE=========")
      print("========USGS, FLAGSTAFF, ARIZONA===========")
      print("========REVISION 3.0.5, DEC 2017===========")
      print("===========================================")

   # exit program if no input folder given
   if not image:
      print('An image file is required!!!!!!')
      sys.exit(2)

   # print given arguments to screen and convert data type where necessary
   if image:
      print('Input image is '+image)

   if density:
      density = np.asarray(density,int)
      print('Every '+str(density)+' rows will be processed')

   if resolution:
      resolution = np.asarray(resolution,float)
      print('Resolution is '+str(resolution))

   if dofilter:
      dofilter = np.asarray(dofilter,int)
      if dofilter==1:
         print('Image will be filtered')
      else:
         print('Image will not be filtered')

   if maxscale:
      maxscale = np.asarray(maxscale,int)
      print('Max scale as inverse fraction of data length: '+str(maxscale))

   if notes:
      notes = np.asarray(notes,int)
      print('Analysis of %s sub-octaves per octave: '+str(notes))

   if x:
      x = np.asarray(x, float)
      print('Area to volume conversion constant = '+str(x))

   # ======= stage 1 ==========================
   # read image
   if verbose==1:
      print("~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
      print('Processing image '+image)   
   try:
       #im = imopen(image, flatten=1).astype('uint8')#.convert("L")
       #im = imread.imload(image, as_grey=True).astype('uint8')
       im = imread(image)[:,:,:3] # read image up to 3 layers
       im = np.squeeze(im) # squeeze singleton dimensions
       if len(np.shape(im))==3: # if rgb, convert to grey
          im = (0.299 * im[:,:,0] + 0.5870*im[:,:,1] + 0.114*im[:,:,2]).astype('uint8')

       nx,ny = np.shape(im)
       if nx>ny:
          im=im.T

   except: # IOError:
       print('cannot open '+image)
       sys.exit(2)
       #im = imread(image)

       #nx,ny = np.shape(im)
       #if nx>ny:
       #   im=im.T

   # convert to numpy array
   region = np.array(im)
   #nx, ny = np.shape(region)
   #mn = min(nx,ny)

   # ======= stage 2 ==========================
   # if requested, call sgolay to filter image
   if dofilter==1:
      useregion = filter_me(region) #, mn, nx, ny)

   else: #no filtering
      useregion = rescale(region,0,255)

   # ======= stage 3 ==========================
   # call cwt to get particle size distribution

   #while (np.shape(useregion)[0] / density) > 100:
   #   density = density+1

   d, scales = get_me(useregion, maxscale, notes, density) #mult

   d = d/np.sum(d)
   d = d/(scales**0.5)
   d = d/np.sum(d)

   # ======= stage 4 ==========================
   # trim particle size bins
   index = np.nonzero(scales<4*ny/maxscale)
   scales = scales[index]
   d = d[index]
   d = d/np.sum(d)

   index = np.nonzero(scales>np.pi*2)
   scales = scales[index]
   d = d[index]
   d = d/np.sum(d)
     
   n = np.r_[0:len(scales)]-(len(scales)-1)/2
   d = d*np.exp(-(0.5)*((np.pi/2)*n/((len(scales)-1)/2))**2)
   d = d/np.sum(d)   

   # get real scales by multiplying by resolution (mm/pixel)
   scales = scales*resolution

   # area-by-number to volume-by-number
   r_v = (d*scales**x) / np.sum(d*scales**x) #volume-by-weight proportion

   # ======= stage 5 ==========================
   # calc particle size stats
   mnsz = np.sum(r_v*scales)
   if verbose==1:
      print("mean size = "+str(mnsz)) 

   srt = np.sqrt(np.sum(d*((scales-mnsz)**2)))
   if verbose==1:
      print("stdev = "+str(srt))

   sk = (sum(d*((scales-mnsz)**3)))/(100*srt**3)
   if verbose==1:
      print("skewness = "+str(sk))

   kurt = (sum(d*((scales-mnsz)**4)))/(100*srt**4)
   if verbose==1:
      print("kurtosis = "+str(kurt))

   pd = np.interp([.05,.1,.16,.25,.5,.75,.84,.9,.95],np.hstack((0,np.cumsum(d))), np.hstack((0,scales)) )

   # ======= stage 6 ==========================
   # return a dict object of stats
   return {'mean grain size': mnsz, 'grain size sorting': srt, 'grain size skewness': sk, 'grain size kurtosis': kurt, 'percentiles': [.05,.1,.16,.25,.5,.75,.84,.9,.95], 'percentile_values': pd, 'grain size frequencies': r_v, 'grain size bins': scales}


# =========================================================
# =========================================================
if __name__ == '__main__':

   dgs(image, density=10, resolution=1, dofilter=1, maxscale=8, notes=8, verbose=0, x=0)





# encoding: utf-8
"""
pyDGS - a Python framework for wavelet-based digital grain size analysis

pyDGS is an open-source project dedicated to provide a Python framework to compute estimates of grain size distribution  using the continuous wavelet transform method of Buscombe (2013) from an image of sediment where grains are clearly resolved. DOES NOT REQUIRE CALIBRATION

This program implements the algorithm of:

Buscombe, D. (2013) Transferable Wavelet Method for Grain-Size Distribution from Images of Sediment Surfaces and Thin Sections, and Other Natural Granular Patterns. Sedimentology 60, 1709-1732

https://www.danielbuscombe.com/s/Buscombe_2013_sedimentology_101111-sed12049.pdf

 Author:  Daniel Buscombe
           Northern Arizona University
           Flagstaff, AZ 86001
           daniel.buscombe@nau.edu
 Revision Dec 21, 2017
 First Revision January 18 2013

For more information visit https://github.com/dbuscombe-usgs/pyDGS

https://www.danielbuscombe.com/s/Buscombe_2013_sedimentology_101111-sed12049.pdf

Please contact:
daniel.buscombe@nau.edu

:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
    
    This software is in the public domain because it contains materials that
    originally came from the United States Geological Survey, an agency of the
    United States Department of Interior. For more information, 
    see the official USGS copyright policy at
    http://www.usgs.gov/visual-id/credit_usgs.html#copyright
    Any use of trade, product, or firm names is for descriptive purposes only 
    and does not imply endorsement by the U.S. government.
    
"""

__version__ = '3.0.9'

#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------

from DGS._dgs_class_web import dgs
from DGS.test import *





"""
 Author:  Daniel Buscombe
           Grand Canyon Monitoring and Research Center
           United States Geological Survey
           Flagstaff, AZ 86001
           dbuscombe@usgs.gov
 First Revision January 18 2013
 Revision Oct 27, 2015

For more information visit https://github.com/dbuscombe-usgs/DGS-python

    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
    
    This software is in the public domain because it contains materials that
    originally came from the United States Geological Survey, an agency of the
    United States Department of Interior. For more information, 
    see the official USGS copyright policy at
    http://www.usgs.gov/visual-id/credit_usgs.html#copyright
    Any use of trade, product, or firm names is for descriptive purposes only 
    and does not imply endorsement by the U.S. government.
"""

from __future__ import division
import numpy as np
cimport numpy as np
cimport cython
import scipy.signal as sp

from libc.math cimport abs

# =========================================================
cdef class sgolay2d:
            
    cdef object data
    
    # =========================================================
    @cython.boundscheck(False)
    @cython.cdivision(True)
    @cython.wraparound(False)
    @cython.nonecheck(False)
    def __cinit__(self, np.ndarray[np.uint8_t, ndim=2] z, int window_size, int order ):
       """
       do 2d filtering on matrix
       from http://www.scipy.org/Cookbook/SavitzkyGolay
       """

       # number of terms in the polynomial expression
       cdef float n_terms = ( order + 1 ) * ( order + 2)  / 2.0

       if  window_size % 2 == 0:
           raise ValueError('window_size must be odd')

       if window_size**2 < n_terms:
           raise ValueError('order is too high for the window size')

       cdef int half_size = window_size // 2

       # exponents of the polynomial. 
       # p(x,y) = a0 + a1*x + a2*y + a3*x^2 + a4*y^2 + a5*x*y + ... 
       # this line gives a list of two item tuple. Each tuple contains 
       # the exponents of the k-th term. First element of tuple is for x
       # second element for y.
       # Ex. exps = [(0,0), (1,0), (0,1), (2,0), (1,1), (0,2), ...]
       cdef list exps = [] 
       exps = [ (k-n, n) for k in xrange(order+1) for n in xrange(k+1) ]

       # coordinates of points
       cdef np.ndarray[np.float64_t,ndim=1] ind = np.arange(-half_size, half_size+1, dtype=np.float64)
       cdef np.ndarray[np.float64_t,ndim=1] dx = np.empty(len(ind)*window_size, dtype=np.float64) 
       dx = np.repeat( ind, window_size )
       cdef np.ndarray[np.float64_t,ndim=1] dy = np.empty(len(ind)*window_size, dtype=np.float64) 
       dy = np.tile( ind, [window_size, 1]).reshape(window_size**2, )

       # build matrix of system of equation
       cdef np.ndarray[np.float64_t,ndim=2] A = np.empty( (window_size**2, len(exps)), dtype=np.float64 )
       for i, exp in enumerate( exps ):
          A[:,i] = (dx**exp[0]) * (dy**exp[1])

       # pad input array with appropriate values at the four borders
       cdef np.ndarray[np.float64_t,ndim=2] Z = np.zeros( (z.shape[0] + 2*half_size, z.shape[1] + 2*half_size), dtype=np.float64 )
       # top band
       #Z[:half_size, half_size:-half_size] =  z[0, :] -  abs( np.flipud( z[1:half_size+1, :] ) - z[0, :] )
       Z[:half_size, half_size:-half_size] =  z[0, :] -  abs( z[1:half_size+1, :][::-1,...] - z[0, :] )

       # bottom band
       #Z[-half_size:, half_size:-half_size] = z[-1, :]  + abs( np.flipud( z[-half_size-1:-1, :] )  -z[-1, :] )
       Z[-half_size:, half_size:-half_size] = z[-1, :]  + abs( z[-half_size-1:-1, :][::-1,...]  -z[-1, :] )

       # left band
       #Z[half_size:-half_size, :half_size] = np.tile( z[:,0].reshape(-1,1), [1,half_size]) - abs( np.fliplr( z[:, 1:half_size+1] ) - np.tile( z[:,0].reshape(-1,1), [1,half_size]) )
       Z[half_size:-half_size, :half_size] = np.tile( z[:,0].reshape(-1,1), [1,half_size]) - abs( z[:, 1:half_size+1][:,::-1] - np.tile( z[:,0].reshape(-1,1), [1,half_size]) )


       # right band
       #Z[half_size:-half_size, -half_size:] =  np.tile( z[:,-1].reshape(-1,1), [1,half_size] ) + abs( np.fliplr( z[:, -half_size-1:-1] ) - np.tile( z[:,-1].reshape(-1,1), [1,half_size] ) )
       Z[half_size:-half_size, -half_size:] =  np.tile( z[:,-1].reshape(-1,1), [1,half_size] ) + abs( z[:, -half_size-1:-1][:,::-1] - np.tile( z[:,-1].reshape(-1,1), [1,half_size] ) )

       # central band
       Z[half_size:-half_size, half_size:-half_size] = z

       # top left corner
       #Z[:half_size,:half_size] = z[0,0] - abs( np.flipud(np.fliplr(z[1:half_size+1,1:half_size+1]) ) - z[0,0] )
       #Z[:half_size,:half_size] = abs( np.fliplr(z[1:half_size+1,1:half_size+1])[::-1,...] )
       Z[:half_size,:half_size] = abs( z[1:half_size+1,1:half_size+1][:,::-1][::-1,...] )

       # seg faults here
       # bottom right corner
       #Z[-half_size:,-half_size:] = z[-1,-1] + np.abs( np.flipud(np.fliplr(z[-half_size-1:-1,-half_size-1:-1]) ) - z[-1,-1] )
       Z[-half_size:,-half_size:] =  abs( z[-half_size-1:-1,-half_size-1:-1][:,::-1][::-1,...]  )

       # top right corner
       #Z[:half_size,-half_size:] = Z[half_size,-half_size:] - abs( np.flipud(Z[half_size+1:2*half_size+1,-half_size:]) - Z[half_size,-half_size:] )
       Z[:half_size,-half_size:] = Z[half_size,-half_size:] - abs( Z[half_size+1:2*half_size+1,-half_size:][::-1,...] - Z[half_size,-half_size:] )

       # bottom left corner
       #Z[-half_size:,:half_size] = Z[-half_size:,half_size].reshape(-1,1) - abs( np.fliplr(Z[-half_size:, half_size+1:2*half_size+1]) - Z[-half_size:,half_size].reshape(-1,1) )
       Z[-half_size:,:half_size] = Z[-half_size:,half_size].reshape(-1,1) - abs( Z[-half_size:, half_size+1:2*half_size+1][:,::-1] - Z[-half_size:,half_size].reshape(-1,1) )

       cdef np.ndarray[np.float64_t,ndim=2] m = np.zeros((window_size, window_size), dtype=np.float64)

       # solve system and convolve
       m = np.linalg.pinv(A)[0].reshape((window_size, -1))
       cdef np.ndarray[np.float64_t,ndim=2] out = np.zeros( np.shape(z), dtype=np.float64 )
       out = sp.fftconvolve(Z.astype('f'), m.astype('f'), mode='valid')

       self.data = out
       return

    # =========================================================    
    @cython.boundscheck(False)
    @cython.cdivision(True)
    @cython.wraparound(False)
    @cython.nonecheck(False)
    cpdef np.ndarray getdata(self):
       return self.data
       
       




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
pyDGS - a Python framework for wavelet-based digital grain size analysis

pyDGS is an open-source project dedicated to provide a Python framework to compute estimates of grain size distribution  using the continuous wavelet transform method of Buscombe (2013) from an image of sediment where grains are clearly resolved. DOES NOT REQUIRE CALIBRATION

This program implements the algorithm of:

Buscombe, D. (2013) Transferable Wavelet Method for Grain-Size Distribution from Images of Sediment Surfaces and Thin Sections, and Other Natural Granular Patterns. Sedimentology 60, 1709-1732

https://www.danielbuscombe.com/s/Buscombe_2013_sedimentology_101111-sed12049.pdf

 Author:  Daniel Buscombe
           Northern Arizona University
           Flagstaff, AZ 86001
           daniel.buscombe@nau.edu
 Revision Dec 21, 2017
 First Revision January 18 2013

For more information visit https://github.com/dbuscombe-usgs/pyDGS

https://www.danielbuscombe.com/s/Buscombe_2013_sedimentology_101111-sed12049.pdf

:install:
    python setup.py install
    sudo python setup.py install
    pip install pyDGS
    
:test:
    python -c "import DGS; DGS.test.dotest_web()"

    python
    import DGS

    image_file = '/home/sed_images/my_image.png'

    density = 10 # process every 10 lines
    resolution = 0.01 # mm/pixel
    dofilter =1 # filter the image
    notes = 8 # notes per octave
    maxscale = 8 #Max scale as inverse fraction of data length
    verbose = 1 # print stuff to screen
    dgs_stats = DGS.dgs(image_file, density, resolution, dofilter, maxscale, notes, verbose)

 REQUIRED INPUTS:
 simply a single file path
 
 OPTIONAL INPUTS [default values][range of acceptable values]
 density = process every *density* lines of image [10][1 - 100]
 resolution = spatial resolution of image in mm/pixel [1][>0]
 dofilter = spatial resolution of image in mm/pixel [1][0 or 1]
 notes = notes per octave to consider in continuous wavelet transform [8][1 - 8]
 maxscale = maximum scale (pixels) as an inverse function of data (image row) length [8][2 - 40]
 verbose = if 1, print stuff to screen [0][0 or 1]

OUTPUT:
A dictionary objects containing the following key/value pairs:
* mean grain size: arithmetic mean grain size
* grain size sorting: arithmetic standard deviation of grain sizes
* grain size skewness: arithmetic skewness of grain size-distribution
* grain size kurtosis: arithmetic kurtosis of grain-size distribution
* percentiles: 5th, 10th, 16th, 25th, 50th, 75th, 84th, 90th, and 95th percentile of the cumulative grain size (% less than) particle size distribution
* grain size frequencies: the normalised frequencies associated with 'grain size bins'
* grain size bins: grain size values at which the distribution is evaluated


PROCESSING NOTES:
Note that the larger the density parameter, the longer the execution time. 

:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
    
    This software is in the public domain because it contains materials that
    originally came from the United States Geological Survey, an agency of the
    United States Department of Interior. For more information, 
    see the official USGS copyright policy at
    http://www.usgs.gov/visual-id/credit_usgs.html#copyright
    Any use of trade, product, or firm names is for descriptive purposes only 
    and does not imply endorsement by the U.S. government.
    
"""

#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------

import os, sys, glob
import inspect
from distutils.core import setup
from distutils.extension import Extension
#from setuptools import setup, Extension
import numpy as np

# Directory of the current file 
SETUP_DIRECTORY = os.path.dirname(os.path.abspath(inspect.getfile(
    inspect.currentframe())))

# Set this to True to enable building extensions using Cython.
# Set it to False to build extensions from the C file (that
# was previously created using Cython).
# Set it to 'auto' to build with Cython if available, otherwise
# from the C file.
USE_CYTHON = True

if USE_CYTHON:
   try:
      from Cython.Distutils import build_ext
   except:
      USE_CYTHON = False

# Read version from distmesh/__init__.py
with open(os.path.join('DGS', '__init__.py')) as f:
    line = f.readline()
    while not line.startswith('__version__'):
        line = f.readline()
exec(line, globals())

ext_modules = [ ]
cmdclass = { }

if USE_CYTHON:
    ext_modules += [
        Extension("DGS.cwt", [ "DGS/cwt.pyx" ],
        include_dirs=[np.get_include()]),
        Extension("DGS.sgolay", [ "DGS/sgolay.pyx" ],
        include_dirs=[np.get_include()]),
    ]
    cmdclass.update({ 'build_ext': build_ext })
else:
    ext_modules += [
        Extension("DGS.cwt", [ "DGS/cwt.c" ],
        include_dirs=[np.get_include()]),
        Extension("DGS.sgolay", [ "DGS/sgolay.c" ],
        include_dirs=[np.get_include()]),
    ]
install_requires = [
    'numpy','scipy','Pillow','cython',
]

def setupPackage():
   setup(name='pyDGS',
         version=__version__,
         description='wavelet-based digital grain size analysis',
         classifiers=[
             'Intended Audience :: Science/Research',
             'Intended Audience :: Developers',
             'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
             'Programming Language :: Python',
             'Programming Language :: Python :: 2.7',
             'Programming Language :: Cython',
             'Topic :: Scientific/Engineering',
             'Topic :: Scientific/Engineering :: Physics',
         ],
         keywords='sediment',
         author='Daniel Buscombe',
         author_email='dbuscombe@usgs.gov',
         url='https://github.com/dbuscombe-usgs/pyDGS',
         download_url ='https://github.com/dbuscombe-usgs/pyDGS/archive/master.zip',
         install_requires=install_requires,
         license = "GNU GENERAL PUBLIC LICENSE v3",
         packages=['DGS'],
         cmdclass = cmdclass,
         ext_modules=ext_modules,
         platforms='OS Independent',
         include_dirs = [np.get_include()],
         package_data={'DGS': ['*.JPG','*.jpg',]}
   )

if __name__ == '__main__':
    # clean --all does not remove extensions automatically
    if 'clean' in sys.argv and '--all' in sys.argv:
        import shutil
        # delete complete build directory
        path = os.path.join(SETUP_DIRECTORY, 'build')
        try:
            shutil.rmtree(path)
        except:
            pass
        # delete all shared libs from lib directory
        path = os.path.join(SETUP_DIRECTORY, 'DGS')
        for filename in glob.glob(path + os.sep + '*.pyd'):
            try:
                os.remove(filename)
            except:
                pass
        for filename in glob.glob(path + os.sep + '*.so'):
            try:
                os.remove(filename)
            except:
                pass
    setupPackage()




"""
 Author:  Daniel Buscombe
           Northern Arizona University
           Flagstaff, AZ 86001
           daniel.buscombe@nau.edu
 Revision Dec 21, 2017
 First Revision January 18 2013

For more information visit https://github.com/dbuscombe-usgs/pyDGS

https://www.danielbuscombe.com/s/Buscombe_2013_sedimentology_101111-sed12049.pdf

Please contact:
daniel.buscombe@nau.edu

For more information visit https://github.com/dbuscombe-usgs/DGS-python

    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
    
    This software is in the public domain because it contains materials that
    originally came from the United States Geological Survey, an agency of the
    United States Department of Interior. For more information, 
    see the official USGS copyright policy at
    http://www.usgs.gov/visual-id/credit_usgs.html#copyright
    Any use of trade, product, or firm names is for descriptive purposes only 
    and does not imply endorsement by the U.S. government.
"""

#python -c "import DGS; DGS.test.dotest()"
import DGS
import os

__all__ = [
    'dotest',
    'dotest_web',
    ]

def dotest():
   dotest_web()

def dotest_web():
   image= DGS.__path__[0]+os.sep+'IMG_0229.JPG'
   resolution = 1
   density = 10
   dofilter=1
   maxscale=8
   notes=8
   verbose=1
   x = -0.5

   DGS.dgs(image, density, resolution, dofilter, maxscale, notes, verbose, x)

if __name__ == '__main__':
   dotest()







"""
 Author:  Daniel Buscombe
           Grand Canyon Monitoring and Research Center
           United States Geological Survey
           Flagstaff, AZ 86001
           dbuscombe@usgs.gov
 Revision Sept 7, 2015 
 First Revision January 18 2013

For more information visit https://github.com/dbuscombe-usgs/DGS-python

    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
    
    This software is in the public domain because it contains materials that
    originally came from the United States Geological Survey, an agency of the
    United States Department of Interior. For more information, 
    see the official USGS copyright policy at
    http://www.usgs.gov/visual-id/credit_usgs.html#copyright
    Any use of trade, product, or firm names is for descriptive purposes only 
    and does not imply endorsement by the U.S. government.
"""

from __future__ import division
import numpy as np
cimport numpy as np
cimport cython
from libc.math cimport sqrt,log,abs
#from scipy.signal import kaiser

# =========================================================
cdef class Cwt:
    """
    continuous Morlet wavelet transform
    Implements via the Fourier transform
    Returns an instance.
    """
    cdef object data
    cdef object scale
    cdef object notes
    cdef object cwt
    cdef object fftdata
    cdef object nscale
    cdef object scales
    cdef object currentscale
    cdef object win
    cdef object density
    cdef object r
    #cdef object mult
                
    # =========================================================
    @cython.boundscheck(False)
    @cython.cdivision(True)
    @cython.wraparound(False)
    @cython.nonecheck(False)
    def __cinit__(self, np.ndarray[np.int8_t, ndim=2] matrix, int largestscale, int notes, int density): #, float mult):
        """
        Continuous Morlet wavelet transform of data

        data:    data in array to transform
        notes:   number of scale intervals per octave
        largestscale: largest scale as inverse fraction of length
                 of data array
                 scale = len(data)/largestscale
                 smallest scale should be >= 2 for meaningful data
        """
        self.win = np.shape(matrix)[0]
        self.density = density
        #self.mult = mult
                
        cdef float pi = 3.14159265
        
        cdef np.ndarray r
        r = np.arange(1,self.win-1,self.density, dtype=np.int)
        r = r[r<self.win]
        self.r = r
        cdef int lr = len(self.r)
        cdef int i, scaleindex
        
        cdef np.ndarray[np.float64_t, ndim=0] currentscale 
        
        cdef double base2 
        base2 = np.floor(log(self.win)/log(2) + 0.4999)
                   
        cdef int ndata = int(2**(base2+1)) #len(data)
        cdef int tmp = 0
        self.nscale = tmp
        self.scale = largestscale
        self._setscales(ndata,largestscale,notes)
        cdef np.ndarray[np.complex64_t, ndim=3] cwt = np.zeros((self.nscale,ndata,len(self.r)),dtype=np.complex64)
        self.cwt = cwt
        cdef np.ndarray[np.float64_t, ndim=1] omega = np.empty(ndata, dtype=np.float64)
        omega = np.array(range(0,np.int(ndata/2))+range(-np.int(ndata/2),0))*(2.0*pi/ndata)
        
        cdef np.ndarray[np.int8_t,ndim=1] data = np.empty(self.win, dtype=np.int8)
        cdef np.ndarray[np.float64_t,ndim=1] data2 = np.empty(ndata, dtype=np.float64)
        cdef np.ndarray[np.complex128_t,ndim=1] datahat = np.empty(ndata, dtype=np.complex128)
        cdef np.ndarray[np.float64_t,ndim=1] s_omega = np.empty(ndata, dtype=np.float64)
        cdef np.ndarray[np.float64_t,ndim=1] psihat = np.empty(ndata, dtype=np.float64)

        for i from 0 <= i < lr:  
           data = np.asarray( self._column(matrix, np.int(self.r[i]) ) )
           data2 = self._pad2nxtpow2(data - np.mean(data), base2) 
                      
           datahat = np.fft.fft(data2)
           self.fftdata = datahat
      
           for scaleindex from 0 <= scaleindex < self.nscale:
              currentscale = np.asarray(self.scales[scaleindex])
              self.currentscale = currentscale  # for internal use
              s_omega = omega*currentscale
              psihat = self._wf(s_omega) * sqrt(2.0*pi*currentscale)
              self.cwt[scaleindex,0:ndata,i] = np.fft.ifft(psihat * datahat)        
        return

    # =========================================================
    @cython.boundscheck(False)   
    @cython.cdivision(True)
    @cython.wraparound(False)
    @cython.nonecheck(False)
    cpdef int _log2(self, double x):
        '''
        utility function to return (integer) log2
        '''
        return int(log(x+0.0001)/ log(2.0)+0.0001)
        
    # =========================================================
    @cython.boundscheck(False)   
    @cython.cdivision(True)
    @cython.wraparound(False)
    @cython.nonecheck(False)
    cpdef int _setscales(self, int ndata, int largestscale, int notes):
        """
        returns a log scale based on notes per ocave
        """
        #cdef float pi = 3.14159265
        cdef int noctave = self._log2( ndata/largestscale/2 )
        self.nscale = notes*noctave
        cdef np.ndarray[np.float64_t, ndim=1] scales = np.empty(self.nscale,np.float64)
        self.scales = scales
        for j from 0 <= j < self.nscale:
             self.scales[j] = ndata/(self.scale*(2.0**(self.nscale-1-j)/notes))
        return 0
        
    # =========================================================
    @cython.boundscheck(False)   
    @cython.cdivision(True)
    @cython.wraparound(False)
    @cython.nonecheck(False)
    cpdef np.ndarray _wf(self, np.ndarray s_omega):
       """
       Morlet mother wavelet
       """    
       cdef np.ndarray[np.int64_t, ndim=1] H = np.ones(np.shape(s_omega), np.int64)
       return 0.75112554*( np.exp(-(s_omega-6.0)**2/2.0))*H
      
    # =========================================================
    @cython.boundscheck(False)   
    @cython.cdivision(True)
    @cython.wraparound(False)
    @cython.nonecheck(False)
    cpdef np.ndarray _pad2nxtpow2(self, np.ndarray data, double base2):
       """
       zero pad numpy array up to next power 2
       """
       cdef np.ndarray[np.float64_t, ndim=2] Y = np.zeros((1, np.int(2**(base2+1)) ), np.float64)
       Y.flat[np.arange(self.win)] = data
       return np.squeeze(Y)
      
    # =========================================================
    @cython.boundscheck(False)   
    @cython.cdivision(True)
    @cython.wraparound(False)
    @cython.nonecheck(False)
    cpdef list _column(self, np.ndarray matrix, int i):
       """
       return a column from a matrix
       """
       return [row[i] for row in matrix]          

    # =========================================================
    @cython.boundscheck(False)   
    @cython.cdivision(True)
    @cython.wraparound(False)
    @cython.nonecheck(False)
    cpdef np.ndarray _getwave(self):
        """
        return power spectra
        """
        cdef np.ndarray[np.float64_t, ndim=3] wave = np.empty((self.nscale,self.win,len(self.r)), np.float64)
        for i from 0 <= i < len(self.r):  
           wave[:,:,i] = np.tile(self.scales**-1, (self.win,1)).T*(abs(self.cwt[:,0:self.win,i])**2)
        return wave
        
    # =========================================================
    @cython.boundscheck(False)   
    @cython.cdivision(True)
    @cython.wraparound(False)
    @cython.nonecheck(False)
    cpdef np.ndarray getvar(self):
        """
        return variance of wave
        """
        #cdef float pi = 3.14159265
        #cdef np.ndarray n
        #n = np.r_[0:len(self.scales)]-(len(self.scales)-1)/2
        wave = self._getwave()
        
        cdef np.ndarray[np.float64_t, ndim=1] dat= np.empty(len(self.scales), np.float64)
        dat = np.var(np.var(wave.T,axis=1),axis=0)
        dat = dat/np.sum(dat)
        
        #dat = dat*kaiser(len(dat),self.mult)
        #dat = dat/np.sum(dat)
        
        #dat = dat/np.sum(dat) * np.exp(-(0.5)*((pi/2)*n/((len(self.scales)-1)/2))**2)
        #dat = dat/np.sum(dat)
           
        return dat
        
    # =========================================================
    @cython.boundscheck(False)   
    @cython.cdivision(True)
    @cython.wraparound(False)
    @cython.nonecheck(False)
    cpdef np.ndarray getscales(self):
        """
        get scales
        """
        return self.scales      
        
        
